@@ -1,4 +1,4 @@
-CREATE VIEW os_csv_raw AS SELECT *
+CREATE OR REPLACE VIEW os_csv_raw AS SELECT *
     /* csv_folder needs to be replaced with a valid path
     in R use glue::glue() */
     FROM read_csv_auto('{csv_folder}**/*.csv.gz', delim='|', header=TRUE, hive_partitioning=TRUE,
